;|*======================================================================================*
Mass Utitlity
**** BETA 0.1 ***** 
Calculates the mass of multiple selected solids
*======================================================================================*|;

(prompt "\n--------------------------------------------------------------")
(prompt "\nMass of solid objects Version 0.1 (16th July 2006)\n")


;*======================================================================================*
;|
Function to set a command line command for the item numbers function
|;
(defun C:DDMass ()
	(DDMass:Run)
	)
;**================================================


;*======================================================================================*
;|
Main program control function.
Call MASSPROP, recovers volume and initiates dialog box.
|;
(defun DDMASS:Run ( / ename i n selset sumvol v vars vlaobj vol)
	;|Function works by using the VLAX-GET-PROPERTY function of the VLISP extension
	suite for each selected solid, allowing the user to select the objects and
	initiates the dialog box to display the results|;
	
	(setq vars (list))
	(setq vars (append vars (list (cons "CMDECHO" (getvar "CMDECHO")))))
	(setvar "CMDECHO" 0)
	(setq vars (append vars (list (cons "BLIPMODE" (getvar "BLIPMODE")))))
	(setvar "BLIPMODE" 0)
	(setq vars (append vars (list (cons "MENUECHO" (getvar "MENUECHO")))))
	(setvar "MENUECHO" 15)
	(setq vars (append vars (list (cons "CLAYER" (getvar "CLAYER")))))
	(setq vars (append vars (list (cons "SNAPMODE" (getvar "SNAPMODE")))))
	(setq vars (append vars (list (cons "OSMODE" (getvar "OSMODE")))))
	
	;before calling any VLISP entensions need to load the VLISP extension suite
	(vl-load-com)

	;allowing the user to select the solids by creating a selection set
	(setq selset (ssget '((0 . "3DSOLID"))))
	
	;for each selected 3DSOLID recovering the associated volume
	(setq i 0);indexer for referencing the entities in the selection set
	(setq vol (list));initiating the list of volumes foe each entity
	(while (< i (sslength selset))
		;retrieving the entity name of the indexed entity
		(setq ename (cdr (assoc -1 (entget (ssname selset i)))))
		;retreiving the VLA-OBJECT property of the entity
		(setq vlaobj (vlax-ename->vla-object ename))
		;retrieving the volume of each entity in the selection set
		(setq vol (append vol (list (list ename (vlax-get-property vlaobj 'Volume)))))
		(setq i (1+ i));increasing the index number
		)
	
	;adding the volumes together
	(setq sumvol 0.0)
	(foreach v vol (setq sumvol (+ sumvol (cadr v))))
	
	;calling the dialog box for displaying the mass
	(DDMass:HandleDialog sumvol selset)
	
	;setting the end of the UNDO command
	(command ".UNDO" "END")

	(command ".REGENALL")
	(foreach n vars (setvar (car n) (cdr n)))
	
	(print)
	)
;**================================================


;*======================================================================================*
;|
Dialog box handling function.
Loads the densities of the materials and display the dialog box
|;
(defun DDMASS:HandleDialog (sumvol selset / dcl_id dens)
	;loading the dialog box definitions
	(if (/= (findfile "mass.dcl") nil) (setq dcl_id (load_dialog (findfile "mass.dcl"))))
	
	;if the dialog box definition loaded then load densities and display the dialog box
	(if (AND (/= dcl_id nil) (> dcl_id 0))
		(progn
			;load densities
			(setq dens (DDMass:LoadDensities))
			
			;initiating the dialog box
			(if (new_dialog "MassDiag" dcl_id)
				(progn
					;initiating the dialog box
					(setq lst (DDMass:InitiateDialog dens))
					(setq classes (car lst))
					(setq matls (cadr lst))
					;Displaying the output info
					(DDMass:SetOutput sumvol dens classes matls)
					)
				)
			
			;defining the action for the list boxes.
			(action_tile "class" "(setq matls (DDMass:HandleClassClick sumvol dens classes matls))")
			(action_tile "matl" "(DDMass:HandleMatlClick sumvol dens classes matls)")
			;defining the action for the info button
			(action_tile "Info" "(INFO:Show \"Mass Utility Version 0.1\" \"Mass utility\")")
			
			;starting the actual dialog box
			(start_dialog)

			;unloading dialog file
			(unload_dialog dcl_id)
			)
		(progn
			;dialog file not loaded
			(DOSLIBAlert "Could not load dialog box!\n\nFunction unavailable." "Fatal Error" 1 5)
			(prompt "\nCould not load dialog box, exiting")
			)
		)
	)
;**================================================


;*======================================================================================*
;|
Load densities from material file.
|;
(defun DDMASS:LoadDensities ( / dens file_id line)
	;opening the data file and retrieving the densities
	(setq file_id (open (findfile "Material Density.dat") "r"))
	
	;|Loading each line of the file.
	If the first sumbol is ";" then ignore the line|;
	(setq line "d" dens (list))
	;reading the line until the end is reached
	(while (AND (/= line "") (/= line nil))
		;reading line from file
		(setq line (read-line file_id))
		;if file is not nil and the line is not a comment then convert to list
		(if (AND (/= line nil) (/= (substr line 1 1) ";"))
			(setq dens (append dens (list (read (strcat "(" line ")")))))
			)
		)
	
	;closing the material density file
	(close file_id)
	
	dens
	)
;**================================================


;*======================================================================================*
;|
Loading the materials in the dialog box
|;
(defun DDMASS:InitiateDialog (dens / c classes m matls)
	
	;loading the classes from the materials list
	(setq classes (DDMass:LoadClasses dens))
	;loading the materials for the first class
	(setq matls (DDMass:LoadMaterials dens (car classes)))
	
	;adding the classes to the dialog box
	(start_list "class")
	(foreach c classes (add_list c))
	(end_list)
	(set_tile "class" "0")
	;adding the materials to the dialog box
	(start_list "matl")
	(foreach m matls (add_list m))
	(end_list)
	(set_tile "matl" "0")
	
	(list classes matls)
	)
;**================================================


;*======================================================================================*
;|
Loading the classes from the materials list
|;
(defun DDMASS:LoadClasses (dens / classes d)
	;retrieving the material classes from the materials list
	(list classes (list))
	(foreach d dens
		;checking whether the class of the indexed material is already in the list of classes
		(if (= (member (cadr d) classes) nil)
			;adding the class in to the list of classes
			(setq classes (append classes (list (cadr d))))
			)
		)
	
	;sorting the list of classes
	(setq classes (vl-sort classes '<))
	
	classes
	)
;**================================================


;*======================================================================================*
;|
Loading the materials for the referenced class
|;
(defun DDMASS:LoadMaterials (dens class / d matls)
	;retrieving the material classes from the materials list
	(list matls (list))
	(foreach d dens
		;checking whether the class of the indexed material is already in the list of classes
		(if (= (cadr d) class)
			;adding the class in to the list of classes
			(setq matls (append matls (list (car d))))
			)
		)
	
	;sorting the list of classes
	(setq matls (vl-sort matls '<))
	
	matls
	)
;**================================================


;*======================================================================================*
;|
Handle c click in the class list box
|;
(defun DDMASS:HandleClassClick (sumvol dens classes matls / m matls)
	;retrieving the list of materials
	(setq matls (DDMass:LoadMaterials dens (nth (atoi (get_tile "class")) classes)))
	
	;adding the materials to the materials list box
	(start_list "matl")
	(foreach m matls (add_list m))
	(end_list)
	(set_tile "matl" "0")
	
	(DDMass:HandleMatlClick sumvol dens classes matls)
	
	matls
	)
;**================================================


;*======================================================================================*
;|
Handle a click in the material list box.
Just need to reset the output info
|;
(defun DDMASS:HandleMatlClick (sumvol dens classes matls)
	;setting the output info on the dialog box
	(DDMass:SetOutput sumvol dens classes matls)
	)
;**================================================


;*======================================================================================*
;|
Handle a click in the material list box.
Just need to reset the output info
|;
(defun DDMASS:SetOutput (sumvol dens classes matls / density mass masstr volstr)
	;determing the density
	(setq density (last (assoc (nth (atoi (get_tile "matl")) matls) dens)))
	;calculating the mass
	(setq mass (/ (* sumvol density) (* 1000 1000 1000.0)))
	(if (< mass 900.0)
		(setq masstr (strcat "Mass : " (rtos mass 2 2) " kg"))
		(setq masstr (strcat "Mass : " (rtos (/ mass 1000.0) 2 2) " tonne"))
		)
	;setting the volume
	(cond
		((> sumvol (* 1000 1000 900.0)) (setq volstr (strcat "Volume : " (rtos (/ sumvol (* 1000 1000 1000.0)) 2 2) " cu m")))
		((> sumvol 50000.0) (setq volstr (strcat "Volume : " (rtos (/ sumvol (* 10 10 10.0)) 2 2) " cu cm")))
		(T (setq volstr (strcat "Volume : " (rtos sumvol 2 2) " cu mm")))
		)
	
	;loading the output info
	(set_tile "material" (strcat "Material : " (nth (atoi (get_tile "matl")) matls)))
	(set_tile "density" (strcat "Density : " (rtos density 2 0) " kg/cu m"))
	(set_tile "volume" volstr)
	(set_tile "mass" masstr)	
	)
;**================================================


;adding the library files
(if (= (atoms-family 1 '("INFO:Show")) nil) (setq n 0) (load "INFO.lsp"))
(if (= (atoms-family 1 '("DOSLIBAlert")) nil) (setq n 0) (load "DOSLIB Functions.lsp"))


(prompt "\nMass utility loaded.  Use DDMass to activate.")
(prompt "\n--------------------------------------------------------------")
(print)

